@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 __all__ = ['IndexedComponent', 'ActiveIndexedComponent']
 
@@ -306,6 +307,21 @@ class IndexedComponent(Component):
             self._implicit_subsets = tmp
             self._index = tmp[0].cross(*tmp[1:])
 
+    def __getstate__(self):
+        # Special processing of getstate so that we never copy the
+        # UnindexedComponent_set set
+        state = super(IndexedComponent, self).__getstate__()
+        if not self.is_indexed():
+            state['_index'] = None
+        return state
+
+    def __setstate__(self, state):
+        # Special processing of setstate so that we never copy the
+        # UnindexedComponent_set set
+        if state['_index'] is None:
+            state['_index'] = UnindexedComponent_set
+        super(IndexedComponent, self).__setstate__(state)
+
     def to_dense_data(self):
         """TODO"""
         for ndx in self._index:
@@ -314,7 +330,7 @@ class IndexedComponent(Component):
 
     def clear(self):
         """Clear the data in this component"""
-        if UnindexedComponent_set != self._index:
+        if self.is_indexed():
             self._data = {}
         else:
             raise DeveloperError(
@@ -327,11 +343,11 @@ class IndexedComponent(Component):
 
     def is_indexed(self):
         """Return true if this component is indexed"""
-        return UnindexedComponent_set != self._index
+        return self._index is not UnindexedComponent_set
 
     def dim(self):
         """Return the dimension of the index"""
-        if id(UnindexedComponent_set) == id(self._index):
+        if not self.is_indexed():
             return 0
         return getattr(self._index, 'dimen', 0)
 
@@ -641,7 +657,7 @@ the value() function.""" % ( self.name, i ))
 
     def set_value(self, value):
         """Set the value of a scalar component."""
-        if UnindexedComponent_set != self._index:
+        if self.is_indexed():
             raise ValueError(
                 "Cannot set the value for the indexed component '%s' "
                 "without specifying an index value.\n"
