@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 __all__ = ['Param']
 
@@ -73,7 +74,7 @@ class _ParamData(ComponentData, NumericValue):
         """
         if self.value is None:
             raise ValueError(
-                "Error evaluating Param value (%s):\nThe Param value is "
+                "Error evaluating Param value (%s):\n\tThe Param value is "
                 "undefined and no default value is specified"
                 % ( self.name, ))
         return self.value
@@ -107,7 +108,10 @@ class _ParamData(ComponentData, NumericValue):
         if self.value:
             return True
         if self.value is None:
-            raise ValueError("Param: value is undefined")
+            raise ValueError(
+                "Error evaluating Param value (%s):\n\tThe Param value is "
+                "undefined and no default value is specified"
+                % ( self.name, ))
         return False
 
     __bool__ = __nonzero__
@@ -138,7 +142,7 @@ class Param(IndexedComponent):
     def __new__(cls, *args, **kwds):
         if cls != Param:
             return super(Param, cls).__new__(cls)
-        if args == () or (type(args[0]) is set and args[0] == UnindexedComponent_set and len(args)==1):
+        if not args or (args[0] is UnindexedComponent_set and len(args)==1):
             return SimpleParam.__new__(SimpleParam)
         else:
             return IndexedParam.__new__(IndexedParam)
@@ -293,7 +297,8 @@ class Param(IndexedComponent):
         should only be used by developers!
         """
         if not self._mutable:
-            raise RuntimeError("Cannot call store_values method on immutable Param="+ self.name)
+            raise RuntimeError("Cannot call store_values method on "
+                               "immutable Param %s" % (self.name,))
         #
         _srcType = type(new_values)
         _isDict = _srcType is dict or ( \
@@ -345,9 +350,9 @@ class Param(IndexedComponent):
             if _isDict:
                 if None not in new_values:
                     raise RuntimeError(
-                        "Cannot store value for scalar Param="+
-                        self.name+"; no value with index None "
-                        "in input new values map.")
+                        "Cannot store value for scalar Param %s:\n\tNo value "
+                        "with index None in the new values dict."
+                        % (self.name,))
                 new_values = new_values[None]
             # scalars have to be handled differently
             self._data[None] = new_values
@@ -368,7 +373,7 @@ class Param(IndexedComponent):
             else:
                 idx_str = '%s[%s]' % (self.local_name, idx,)
             raise ValueError(
-                "Error retrieving Param value (%s): This parameter has "
+                "Error retrieving Param value (%s):\n\tThis parameter has "
                 "not been constructed" % ( idx_str,) )
         if val is None:
             # If the Param is mutable, then it is OK to create a Param
@@ -387,7 +392,7 @@ class Param(IndexedComponent):
             else:
                 idx_str = '%s' % (self.name,)
             raise ValueError(
-                    "Error retrieving Param value (%s):\nThe Param value is "
+                    "Error retrieving Param value (%s):\n\tThe Param value is "
                     "undefined and no default value is specified"
                     % ( idx_str,) )
         #
@@ -466,8 +471,8 @@ class Param(IndexedComponent):
                 and type(val) in native_types \
                 and val not in self.domain:
             raise ValueError(
-                "Default value (%s) is not valid for Param domain %s" %
-                (str(val), self.domain.name))
+                "Default value (%s) is not valid for Param %s domain %s" %
+                (str(val), self.name, self.domain.name))
         self._default_val = val
 
     def default(self):
@@ -808,8 +813,8 @@ This has resulted in the conversion of the source to dense form.
                 and type(val) in native_types \
                 and val not in self.domain:
             raise ValueError(
-                "Default value (%s) is not valid for Param domain %s" %
-                (str(val), self.domain.name))
+                "Default value (%s) is not valid for Param %s domain %s" %
+                (str(val), self.name, self.domain.name))
         #
         # Step #1: initialize data from rule value
         #
@@ -856,7 +861,9 @@ This has resulted in the conversion of the source to dense form.
         or constraints.
         """
         if not self._mutable:
-            raise RuntimeError("Cannot invoke reconstruct method of immutable param="+self.name)
+            raise RuntimeError(
+                "Cannot invoke reconstruct method of immutable Param %s"
+                % (self.name,))
         IndexedComponent.reconstruct(self, data=data)
 
     def _pprint(self):
@@ -864,8 +871,7 @@ This has resulted in the conversion of the source to dense form.
         Return data that will be printed for this component.
         """
         return ( [("Size", len(self)),
-                  ("Index", self._index \
-                       if self._index != UnindexedComponent_set else None),
+                  ("Index", self._index if self.is_indexed() else None),
                   ("Domain", self.domain.name),
                   ("Default", "(function)" if type(self._default_val) \
                        is types.FunctionType else self._default_val),
@@ -901,8 +907,10 @@ class SimpleParam(_ParamData, Param):
         if self._constructed:
             return _ParamData.__call__(self, exception=exception)
         if exception:
-            raise ValueError( """Evaluating the numeric value of parameter '%s' before the Param has been
-            constructed (there is currently no value to return).""" % self.name )
+            raise ValueError(
+                "Evaluating the numeric value of parameter '%s' before\n\t"
+                "the Param has been constructed (there is currently no "
+                "value to return)." % (self.name,) )
 
     def set_value(self, value):
         if self._constructed and not self._mutable:
@@ -925,8 +933,8 @@ class IndexedParam(Param):
     def __call__(self, exception=True):
         """Compute the value of the parameter"""
         if exception:
-            msg = 'Cannot compute the value of an array of parameters'
-            raise TypeError(msg)
+            raise TypeError('Cannot compute the value of an indexed Param (%s)'
+                            % (self.name,) )
 
 register_component(Param, "Parameter data that is used to define a model instance.")
 
