@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 __all__ = ['SOSConstraint']
 
@@ -149,7 +150,7 @@ class SOSConstraint(ActiveIndexedComponent):
     def __new__(cls, *args, **kwds):
         if cls != SOSConstraint:
             return super(SOSConstraint, cls).__new__(cls)
-        if args == () or (type(args[0]) == set and args[0] == UnindexedComponent_set and len(args)==1):
+        if not args or (args[0] is UnindexedComponent_set and len(args)==1):
             return SimpleSOSConstraint.__new__(SimpleSOSConstraint)
         else:
             return IndexedSOSConstraint.__new__(IndexedSOSConstraint)
@@ -225,11 +226,11 @@ class SOSConstraint(ActiveIndexedComponent):
         self._constructed = True
 
         if self._rule is None:
-            if self._sosSet is None and not None in self._index:
+            if self._sosSet is None and self.is_indexed():
                 if generate_debug_messages:     #pragma:nocover
                     logger.debug("  Cannot construct "+self.name+".  No rule is defined and no SOS sets are defined.")
             else:
-                if None in self._index:
+                if not self.is_indexed():
                     if self._sosSet is None:
                         if getattr(self._sosVars.index_set(), 'ordered', False):
                             _sosSet = {None: list(self._sosVars.index_set())}
@@ -249,7 +250,7 @@ class SOSConstraint(ActiveIndexedComponent):
                         # Check that the sets are ordered.
                         #
                         ordered=False
-                        if type(sosSet) is list or sosSet == UnindexedComponent_set or len(sosSet) == 1:
+                        if type(sosSet) is list or sosSet is UnindexedComponent_set or len(sosSet) == 1:
                             ordered=True
                         if hasattr(sosSet, 'ordered') and sosSet.ordered:
                             ordered=True
