@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 __all__ = ['Var', '_VarData', 'VarList']
 
@@ -433,7 +434,7 @@ class Var(IndexedComponent):
     def __new__(cls, *args, **kwds):
         if cls != Var:
             return super(Var, cls).__new__(cls)
-        if args == () or (type(args[0]) == set and args[0] == UnindexedComponent_set and len(args)==1):
+        if not args or (args[0] is UnindexedComponent_set and len(args)==1):
             return SimpleVar.__new__(SimpleVar)
         else:
             return IndexedVar.__new__(IndexedVar)
@@ -699,8 +700,7 @@ class Var(IndexedComponent):
     def _pprint(self):
         """Print component information."""
         return ( [("Size", len(self)),
-                  ("Index", self._index \
-                       if self._index != UnindexedComponent_set else None),
+                  ("Index", self._index if self.is_indexed() else None),
                   ],
                  iteritems(self._data),
                  ( "Lower","Value","Upper","Fixed","Stale","Domain"),
@@ -866,14 +866,6 @@ class SimpleVar(_GeneralVarData, Var):
 class IndexedVar(Var):
     """An array of variables."""
 
-    # These methods are normally found on the NumericValue
-    # interface, but they are here to trick the expression
-    # system into reporting errors about trying to use
-    # "indexed NumericValue" objects in expressions
-    def as_numeric(self): return self
-    def is_expression(self): return False
-    def is_relational(self): return False
-
     def fix(self, *val):
         """
         Set the fixed indicator to True. Value argument is optional,
