@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 #
 # Unit Tests for Elements of a Model
 #
@@ -14,8 +15,8 @@
 
 import os
 import sys
-from os.path import abspath, dirname
-currdir = dirname(abspath(__file__))+os.sep
+from os.path import abspath, dirname, join
+currdir = dirname(abspath(__file__))
 import pickle
 import pyutilib.th as unittest
 import pyutilib.services
@@ -224,25 +225,31 @@ class Test(unittest.TestCase):
             return expr == 0
         model.c = Constraint(rule=c_rule)
         opt = SolverFactory('glpk')
-        results = opt.solve(model, keepfiles=True, symbolic_solver_labels=True)
+        results = opt.solve(model, symbolic_solver_labels=True)
         model.solutions.store_to(results)
-        results.write(filename=currdir+"solve1.out", format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve1.out",currdir+"solve1.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,"solve1.out"), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve1.out"), join(currdir,"solve1.txt"),
+            tolerance=1e-4)
         #
         def d_rule(model):
             return model.x[1] >= 0
         model.d = Constraint(rule=d_rule)
         model.d.deactivate()
-        results = opt.solve(model, keepfiles=True)
+        results = opt.solve(model)
         model.solutions.store_to(results)
-        results.write(filename=currdir+"solve1x.out", format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve1x.out",currdir+"solve1.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,"solve1x.out"), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve1x.out"), join(currdir,"solve1.txt"),
+            tolerance=1e-4)
         #
         model.d.activate()
-        results = opt.solve(model, keepfiles=True)
+        results = opt.solve(model)
         model.solutions.store_to(results)
-        results.write(filename=currdir+"solve1a.out", format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve1a.out",currdir+"solve1a.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,"solve1a.out"), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve1a.out"), join(currdir,"solve1a.txt"),
+            tolerance=1e-4)
         #
         model.d.deactivate()
         def e_rule(model, i):
@@ -250,12 +257,22 @@ class Test(unittest.TestCase):
         model.e = Constraint(model.A, rule=e_rule)
         for i in model.A:
             model.e[i].deactivate()
-        results = opt.solve(model, keepfiles=True)
+        results = opt.solve(model)
         model.solutions.store_to(results)
-        results.write(filename=currdir+"solve1b.out", format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve1b.out",currdir+"solve1b.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,"solve1y.out"), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve1y.out"), join(currdir,"solve1.txt"),
+            tolerance=1e-4)
+        #
+        model.e.activate()
+        results = opt.solve(model)
+        model.solutions.store_to(results)
+        results.write(filename=join(currdir,"solve1b.out"), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve1b.out"), join(currdir,"solve1b.txt"),
+            tolerance=1e-4)
 
-    def test_solve3(self):
+    def test_display(self):
         model = ConcreteModel()
         model.A = RangeSet(1,4)
         model.x = Var(model.A, bounds=(-1,1))
@@ -265,8 +282,9 @@ class Test(unittest.TestCase):
                 expr += model.x[i]
             return expr
         model.obj = Objective(rule=obj_rule)
-        model.display(currdir+"solve3.out")
-        self.assertFileEqualsBaseline(currdir+"solve3.out",currdir+"solve3.txt")
+        model.display(join(currdir,"solve3.out"))
+        self.assertFileEqualsBaseline(
+            join(currdir,"solve3.out"), join(currdir,"solve3.txt"))
 
     @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
     def test_solve4(self):
@@ -285,8 +303,10 @@ class Test(unittest.TestCase):
         opt = SolverFactory('glpk')
         results = opt.solve(model, symbolic_solver_labels=True)
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve4.out', format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve4.out",currdir+"solve1.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,'solve4.out'), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve4.out"), join(currdir,"solve1.txt"),
+            tolerance=1e-4)
 
     @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
     def test_solve6(self):
@@ -312,13 +332,16 @@ class Test(unittest.TestCase):
         opt = SolverFactory('glpk')
         results = opt.solve(model, symbolic_solver_labels=True)
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve6.out', format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve6.out", currdir+"solve6.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,'solve6.out'), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve6.out"), join(currdir,"solve6.txt"),
+            tolerance=1e-4)
 
     @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
     def test_solve7(self):
         #
-        # Test that solution values are writen with appropriate quotations in results
+        # Test that solution values are writen with appropriate
+        # quotations in results
         #
         model = ConcreteModel()
         model.y = Var(bounds=(-1,1))
@@ -339,8 +362,10 @@ class Test(unittest.TestCase):
         results = opt.solve(model, symbolic_solver_labels=True)
         #model.display()
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve7.out', format='json')
-        self.assertMatchesJsonBaseline(currdir+"solve7.out", currdir+"solve7.txt", tolerance=1e-4)
+        results.write(filename=join(currdir,'solve7.out'), format='json')
+        self.assertMatchesJsonBaseline(
+            join(currdir,"solve7.out"), join(currdir,"solve7.txt"),
+            tolerance=1e-4)
 
     def test_stats1(self):
         model = ConcreteModel()
@@ -436,6 +461,68 @@ class Test(unittest.TestCase):
         self.assertEqual(tmodel.solutions[0].status, SolutionStatus.feasible)
         self.assertEqual(tmodel.solutions[0].message, None)
 
+    @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
+    def test_solve_with_pickle_then_clone(self):
+        # This tests github issue Pyomo-#65
+        model = ConcreteModel()
+        model.A = RangeSet(1,4)
+        model.b = Block()
+        model.b.x = Var(model.A, bounds=(-1,1))
+        model.b.obj = Objective(expr=summation(model.b.x))
+        model.c = Constraint(expr=model.b.x[1] >= 0)
+        opt = SolverFactory('glpk')
+        self.assertEqual(len(model.solutions), 0)
+        results = opt.solve(model, symbolic_solver_labels=True)
+        self.assertEqual(len(model.solutions), 1)
+        #
+        self.assertEqual(model.solutions[0].gap, 0.0)
+        self.assertEqual(model.solutions[0].status, SolutionStatus.feasible)
+        self.assertEqual(model.solutions[0].message, None)
+        #
+        buf = pickle.dumps(model)
+        tmodel = pickle.loads(buf)
+        self.assertEqual(len(tmodel.solutions), 1)
+        self.assertEqual(tmodel.solutions[0].gap, 0.0)
+        self.assertEqual(tmodel.solutions[0].status, SolutionStatus.feasible)
+        self.assertEqual(tmodel.solutions[0].message, None)
+        self.assertIn(id(tmodel.b.obj), tmodel.solutions[0]._entry['objective'])
+        self.assertIs(
+            tmodel.b.obj,
+            tmodel.solutions[0]._entry['objective'][id(tmodel.b.obj)][0]() )
+
+        inst = tmodel.clone()
+
+        # make sure the clone has all the attributes
+        self.assertTrue(hasattr(inst,'A'))
+        self.assertTrue(hasattr(inst,'b'))
+        self.assertTrue(hasattr(inst.b,'x'))
+        self.assertTrue(hasattr(inst.b,'obj'))
+        self.assertTrue(hasattr(inst,'c'))
+        # and that they were all copied
+        self.assertIsNot(inst.A, tmodel.A)
+        self.assertIsNot(inst.b, tmodel.b)
+        self.assertIsNot(inst.b.x, tmodel.b.x)
+        self.assertIsNot(inst.b.obj, tmodel.b.obj)
+        self.assertIsNot(inst.c, tmodel.c)
+
+        # Make sure the solution is on the new model
+        self.assertTrue(hasattr(inst,'solutions'))
+        self.assertEqual(len(inst.solutions), 1)
+        self.assertEqual(inst.solutions[0].gap, 0.0)
+        self.assertEqual(inst.solutions[0].status, SolutionStatus.feasible)
+        self.assertEqual(inst.solutions[0].message, None)
+
+        # Spot-check some components and make sure all the weakrefs in
+        # the ModelSOlution got updated
+        self.assertIn(id(inst.b.obj), inst.solutions[0]._entry['objective'])
+        _obj = inst.solutions[0]._entry['objective'][id(inst.b.obj)]
+        self.assertIs(_obj[0](), inst.b.obj)
+
+        for v in [1,2,3,4]:
+            self.assertIn(id(inst.b.x[v]), inst.solutions[0]._entry['variable'])
+            _v = inst.solutions[0]._entry['variable'][id(inst.b.x[v])]
+            self.assertIs(_v[0](), inst.b.x[v])
+
     @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
     @unittest.skipIf(not yaml_available, "YAML not available available")
     def test_solve_with_store1(self):
@@ -449,12 +536,18 @@ class Test(unittest.TestCase):
         opt = SolverFactory('glpk')
         results = opt.solve(model, symbolic_solver_labels=True)
         #
-        results.write(filename=currdir+'solve_with_store1.out', format='yaml')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store1.out", currdir+"solve_with_store1.txt")
+        results.write(filename=join(currdir,'solve_with_store1.out'),
+                      format='yaml')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store1.out"),
+            join(currdir,"solve_with_store1.txt"))
         model.solutions.store_to(results)
         #
-        results.write(filename=currdir+'solve_with_store2.out', format='yaml')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store2.out", currdir+"solve_with_store2.txt")
+        results.write(filename=join(currdir,'solve_with_store2.out'),
+                      format='yaml')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store2.out"),
+            join(currdir,"solve_with_store2.txt"))
         #
         # Load results with string indices
         #
@@ -481,12 +574,18 @@ class Test(unittest.TestCase):
         opt = SolverFactory('glpk')
         results = opt.solve(model, symbolic_solver_labels=False)
         #
-        results.write(filename=currdir+'solve_with_store1.out', format='yaml')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store1.out", currdir+"solve_with_store1.txt")
+        results.write(filename=join(currdir,'solve_with_store1.out'),
+                      format='yaml')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store1.out"),
+            join(currdir,"solve_with_store1.txt"))
         model.solutions.store_to(results)
         #
-        results.write(filename=currdir+'solve_with_store2.out', format='yaml')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store2.out", currdir+"solve_with_store2.txt")
+        results.write(filename=join(currdir,'solve_with_store2.out'),
+                      format='yaml')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store2.out"),
+            join(currdir,"solve_with_store2.txt"))
         #
         # Load results with string indices
         #
@@ -512,19 +611,28 @@ class Test(unittest.TestCase):
         opt = SolverFactory('glpk')
         results = opt.solve(model)
         #
-        results.write(filename=currdir+'solve_with_store3.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store3.out", currdir+"solve_with_store3.txt")
+        results.write(filename=join(currdir,'solve_with_store3.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store3.out"),
+            join(currdir,"solve_with_store3.txt"))
         #
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve_with_store4.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store4.out", currdir+"solve_with_store4.txt")
+        results.write(filename=join(currdir,'solve_with_store4.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store4.out"),
+            join(currdir,"solve_with_store4.txt"))
         #
         # Test that we can pickle the results object
         #
         buf = pickle.dumps(results)
         results_ = pickle.loads(buf)
-        results.write(filename=currdir+'solve_with_store4.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store4.out", currdir+"solve_with_store4.txt")
+        results.write(filename=join(currdir,'solve_with_store4.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store4.out"),
+            join(currdir,"solve_with_store4.txt"))
         #
         # Load results with string indices
         #
@@ -551,16 +659,22 @@ class Test(unittest.TestCase):
         results = opt.solve(model)
         #
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve_with_store5.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store5.out", currdir+"solve_with_store4.txt")
+        results.write(filename=join(currdir,'solve_with_store5.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store5.out"),
+            join(currdir,"solve_with_store4.txt"))
         #
         model.solutions.store_to(results, cuid=True)
         buf = pickle.dumps(results)
         results_ = pickle.loads(buf)
         model.solutions.load_from(results_)
         model.solutions.store_to(results_)
-        results_.write(filename=currdir+'solve_with_store6.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store6.out", currdir+"solve_with_store4.txt")
+        results_.write(filename=join(currdir,'solve_with_store6.out'),
+                       format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store6.out"),
+            join(currdir,"solve_with_store4.txt"))
         #
         # Load results with string indices
         #
@@ -574,8 +688,11 @@ class Test(unittest.TestCase):
         tmodel.solutions.load_from(results)
         self.assertEqual(len(tmodel.solutions), 1)
         tmodel.solutions.store_to(results)
-        results.write(filename=currdir+'solve_with_store7.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store7.out", currdir+"solve_with_store4.txt")
+        results.write(filename=join(currdir,'solve_with_store7.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store7.out"),
+            join(currdir,"solve_with_store4.txt"))
 
     @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
     @unittest.skipIf(not yaml_available, "YAML not available available")
@@ -595,8 +712,11 @@ class Test(unittest.TestCase):
         self.assertEqual(len(results.solution), 1)
         #
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve_with_store8.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store8.out", currdir+"solve_with_store4.txt")
+        results.write(filename=join(currdir,'solve_with_store8.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store8.out"),
+            join(currdir,"solve_with_store4.txt"))
 
     @unittest.skipIf(not 'glpk' in solvers, "glpk solver is not available")
     @unittest.skipIf(not yaml_available, "YAML not available available")
@@ -618,8 +738,11 @@ class Test(unittest.TestCase):
         self.assertEqual(len(results.solution), 1)
         #
         model.solutions.store_to(results)
-        results.write(filename=currdir+'solve_with_store8.out', format='json')
-        self.assertMatchesYamlBaseline(currdir+"solve_with_store8.out", currdir+"solve_with_store4.txt")
+        results.write(filename=join(currdir,'solve_with_store8.out'),
+                      format='json')
+        self.assertMatchesYamlBaseline(
+            join(currdir,"solve_with_store8.out"),
+            join(currdir,"solve_with_store4.txt"))
 
 
     def test_create_concrete_from_rule(self):
