@@ -1,20 +1,22 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 import logging
 
 from pyomo.core import *
 from pyomo.core.base.indexed_component import IndexedComponent
 from pyomo.core.base.misc import apply_indexed_rule
+from pyomo.core.base.block import _BlockData, IndexedBlock
 from pyomo.dae import *
 
-from six import iterkeys, itervalues
+from six import iterkeys, itervalues, iteritems
 
 logger = logging.getLogger('pyomo.core')
 
@@ -98,12 +100,8 @@ def update_contset_indexed_component(comp):
     Update any model components which are indexed by a ContinuousSet
     that has changed
     """
-    # FIXME: This implementation is a hack until Var and Constraint get
-    # moved over to Indexed_Component. The update methods below are
-    # roughly what the '_default' method in Var and Constraint should
-    # do when they get reimplemented.
 
-    # Additionally, this implemenation will *NOT* check for or update
+    # This implemenation will *NOT* check for or update
     # components which use a ContinuousSet implicitly. ex) an
     # objective function which iterates through a ContinuousSet and
     # sums the squared error.  If you use a ContinuousSet implicitly
@@ -112,31 +110,52 @@ def update_contset_indexed_component(comp):
 
     if comp.type() is Suffix:
         return
-    if comp.dim() == 1:
-        if comp._index.type() == ContinuousSet:
-            if comp._index.get_changed():
-                if isinstance(comp, Var):
-                    _update_var(comp)
-                elif comp.type() == Constraint:
-                    _update_constraint(comp)
-                elif comp.type() == Expression:
-                    _update_expression(comp)
-    elif comp.dim() > 1:
+    
+    # Params indexed by a ContinuousSet should include an initialize
+    # and/or default rule which will be called automatically when the
+    # parameter value at a new point in the ContinuousSet is
+    # requested. Therefore, no special processing is required for
+    # Params.
+    if comp.type() is Param:
+        return
+
+    # Components indexed by a ContinuousSet must have a dimension of at
+    # least 1
+    if comp.dim() == 0:
+        return
+
+    # Extract the indexing sets. Must treat components with a single
+    # index separately from components with multiple indexing sets.
+    if comp._implicit_subsets is None:
+        indexset = [comp._index]
+    else:
         indexset = comp._implicit_subsets
 
-        for s in indexset:
-            if s.type() == ContinuousSet and s.get_changed():
-                if isinstance(comp, Var): # Don't use the type() method here because we want to catch
-                    _update_var(comp)     # DerivativeVar components as well as Var components
-                elif comp.type() == Constraint:
-                    _update_constraint(comp)
-                elif comp.type() == Expression:
-                    _update_expression(comp)
+    for s in indexset:
+        if s.type() == ContinuousSet and s.get_changed():
+            if isinstance(comp, Var): # Don't use the type() method here because we want to catch
+                _update_var(comp)     # DerivativeVar components as well as Var components
+            elif comp.type() == Constraint:
+                _update_constraint(comp)
+            elif comp.type() == Expression:
+                _update_expression(comp)
+            elif isinstance(comp, Piecewise):
+                _update_piecewise(comp)
+            elif comp.type() == Block: 
+                _update_block(comp)    
+            else:
+                raise TypeError("Found component %s of type %s indexed "\
+                    "by a ContinuousSet. Components of this type are "\
+                    "not currently supported by the automatic "\
+                    "discretization transformation in pyomo.dae. "\
+                    "Try adding the component to the model "\
+                    "after discretizing. Alert the pyomo developers "\
+                    "for more assistance." %(str(comp),comp.type()))
 
 def _update_var(v):
     """
     This method will construct any additional indices in a variable
-    resulting from the discretization of ContinuousSet
+    resulting from the discretization of a ContinuousSet.
     """
 
     # Note: This is not required it is handled by the _default method on
@@ -150,7 +169,7 @@ def _update_var(v):
 def _update_constraint(con):
     """
     This method will construct any additional indices in a constraint
-    resulting from the discretization.
+    resulting from the discretization of a ContinuousSet.
     """
 
     _rule=con.rule
@@ -162,8 +181,8 @@ def _update_constraint(con):
 
 def _update_expression(expre):
     """
-    This method will construct any additional indices in a expression
-    resulting from the discretization
+    This method will construct any additional indices in an expression
+    resulting from the discretization of a ContinuousSet.
     """
     _rule=expre._init_rule
     _parent=expre._parent()
@@ -172,6 +191,66 @@ def _update_expression(expre):
             # Code taken from the construct() method of Expression
             expre.add(i,apply_indexed_rule(expre,_rule,_parent,i))
 
+def _update_block(blk):
+    """
+    This method will construct any additional indices in a block
+    resulting from the discretization of a ContinuousSet. For
+    Block-derived components we check if the Block construct method has
+    been overridden. If not then we update it like a regular block. If
+    construct has been overridden then we try to call the component's
+    update_after_discretization method. If the component hasn't
+    implemented this method then we throw a warning and try to update it
+    like a normal block. The issue, when construct is overridden, is that
+    anything could be happening and we can't automatically assume that
+    treating the block-derived component like a normal block will be
+    sufficient to update it correctly.
+
+    """
+    
+    # Check if Block construct method is overridden
+    # getattr needed below for Python 2, 3 compatibility
+    if blk.construct.__func__ is not getattr(IndexedBlock.construct,
+                                             '__func__', IndexedBlock.construct):
+        # check for custom update function
+        try :
+            blk.update_after_discretization()
+            return
+        except AttributeError:
+            logger.warning('DAE(misc): Attempting to apply a discretization transformation to '
+                  'the Block-derived component "%s". The component overrides the '
+                  'Block construct method but no update_after_discretization() '
+                  'function was found. Will attempt to update as a standard Block '
+                  'but user should verify that the component was expanded correctly. '
+                  'To suppress this warning, please provide an '
+                  'update_after_discretization() function on Block-derived '
+                  'components that override construct()' %(blk.name))            
+
+    # Code taken from the construct() method of Block
+    missing_idx = set(blk._index)-set(iterkeys(blk._data))
+    for idx in list(missing_idx):
+        _block = blk[idx]
+        obj = apply_indexed_rule(
+            None, blk._rule, _block, idx, blk._options )
+ 
+        if isinstance(obj, _BlockData) and obj is not _block:
+            # If the user returns a block, use their block instead
+            # of the empty one we just created.
+            for c in list(obj.component_objects(descend_into=False)):
+                obj.del_component(c)
+                _block.add_component(c.local_name, c)
+                # transfer over any other attributes that are not components
+            for name, val in iteritems(obj.__dict__):
+                if not hasattr(_block, name) and not hasattr(blk, name):
+                    super(_BlockData, _block).__setattr__(name, val)
+
+def _update_piecewise(pw):
+    """
+    This method will construct any additional indices in a Piecewise
+    object resulting from the discretization of a ContinuousSet.
+    """
+    pw._constructed = False
+    pw.construct()
+
 def create_access_function(var):
     """
     This method returns a function that returns a component by calling
