@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 from six.moves import xrange as range
 from six import iteritems, iterkeys
@@ -191,7 +192,7 @@ class BigM_Transformation(Transformation):
                 continue
             c.deactivate()
 
-            name = _name + (cname and '.'+cname or '')
+            name = _name + ('.'+str(cname) if cname is not None else '')
 
             if (not lin_body_map is None) and (not lin_body_map.get(c) is None):
                 raise GDP_Error('GDP(BigM) cannot process linear ' \
