@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 #
 # AMPL Problem Writer Plugin
@@ -990,15 +991,15 @@ class ProblemWriter_nl(AbstractProblemWriter):
 
         for var_ID in LinearVars:
             var = Vars_dict[var_ID]
-            if var.is_integer():
-                LinearVarsInt.add(var_ID)
-            elif var.is_binary():
+            if var.is_binary():
                 L = var.lb
                 U = var.ub
                 if L is None or U is None:
                     raise ValueError("Variable " + str(var.name) +\
                                      "is binary, but does not have lb and ub set")
                 LinearVarsBool.add(var_ID)
+            elif var.is_integer():
+                LinearVarsInt.add(var_ID)
             elif not var.is_continuous():
                 raise TypeError("Invalid domain type for variable with name '%s'. "
                                 "Variable is not continuous, integer, or binary.")
