@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 #
 # Problem Writer for BARON .bar Format Files
@@ -513,7 +514,7 @@ class ProblemWriter_bar(AbstractProblemWriter):
 
                 for param_data in param_data_iter:
                     param_stream = StringIO()
-                    param.to_string(ostream=param_stream, verbose=False)
+                    param_data.to_string(ostream=param_stream, verbose=False)
                     param_string = param_stream.getvalue()
 
                     param_string = ' '+param_string+' '
