@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 #
 # Problem Writer for CPLEX LP Format Files
@@ -822,10 +823,10 @@ class ProblemWriter_cpxlp(AbstractProblemWriter):
 
             # track the number of integer and binary variables, so we know whether
             # to output the general / binary sections below.
-            if vardata.is_integer():
-                integer_vars.append(name_to_output)
-            elif vardata.is_binary():
+            if vardata.is_binary():
                 binary_vars.append(name_to_output)
+            elif vardata.is_integer():
+                integer_vars.append(name_to_output)
             elif not vardata.is_continuous():
                 raise TypeError("Invalid domain type for variable with name '%s'. "
                                 "Variable is not continuous, integer, or binary."
