@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 # TODO: Disabled until we can confirm application to Pyomo models
 import pyomo.solvers.plugins.solvers.ps
@@ -25,17 +26,4 @@ import pyomo.solvers.plugins.solvers.pywrapper
 import pyomo.solvers.plugins.solvers.SCIPAMPL
 import pyomo.solvers.plugins.solvers.CONOPT
 import pyomo.solvers.plugins.solvers.XPRESS
-
-#
-# Interrogate the CBC executable to see if it recognizes the -AMPL flag
-#
-from pyomo.solvers.plugins.solvers.CBCplugin import configure_cbc
-configure_cbc()
-del configure_cbc
-
-#
-# Interrogate the glpsol executable to see if it is new enough to allow the new parser logic
-#
-from pyomo.solvers.plugins.solvers.GLPK import configure_glpk
-configure_glpk()
-del configure_glpk
+import pyomo.solvers.plugins.solvers.IPOPT
